# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from slash_sdk import SlashSDK, AsyncSlashSDK
from tests.utils import assert_matches_type
from slash_sdk.types import Oauth2GetTokenResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestOauth2:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_get_token_overload_1(self, client: SlashSDK) -> None:
        oauth2 = client.oauth2.get_token(
            code="code",
            grant_type="authorization_code",
            redirect_uri="redirect_uri",
        )
        assert_matches_type(Oauth2GetTokenResponse, oauth2, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_get_token_with_all_params_overload_1(self, client: SlashSDK) -> None:
        oauth2 = client.oauth2.get_token(
            code="code",
            grant_type="authorization_code",
            redirect_uri="redirect_uri",
            code_verifier="code_verifier",
            prompt="prompt",
            scope="scope",
        )
        assert_matches_type(Oauth2GetTokenResponse, oauth2, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_get_token_overload_1(self, client: SlashSDK) -> None:
        response = client.oauth2.with_raw_response.get_token(
            code="code",
            grant_type="authorization_code",
            redirect_uri="redirect_uri",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        oauth2 = response.parse()
        assert_matches_type(Oauth2GetTokenResponse, oauth2, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_get_token_overload_1(self, client: SlashSDK) -> None:
        with client.oauth2.with_streaming_response.get_token(
            code="code",
            grant_type="authorization_code",
            redirect_uri="redirect_uri",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            oauth2 = response.parse()
            assert_matches_type(Oauth2GetTokenResponse, oauth2, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_get_token_overload_2(self, client: SlashSDK) -> None:
        oauth2 = client.oauth2.get_token(
            grant_type="refresh_token",
            refresh_token="refresh_token",
        )
        assert_matches_type(Oauth2GetTokenResponse, oauth2, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_get_token_with_all_params_overload_2(self, client: SlashSDK) -> None:
        oauth2 = client.oauth2.get_token(
            grant_type="refresh_token",
            refresh_token="refresh_token",
            code="code",
            code_verifier="code_verifier",
            prompt="prompt",
            redirect_uri="redirect_uri",
            scope="scope",
        )
        assert_matches_type(Oauth2GetTokenResponse, oauth2, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_get_token_overload_2(self, client: SlashSDK) -> None:
        response = client.oauth2.with_raw_response.get_token(
            grant_type="refresh_token",
            refresh_token="refresh_token",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        oauth2 = response.parse()
        assert_matches_type(Oauth2GetTokenResponse, oauth2, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_get_token_overload_2(self, client: SlashSDK) -> None:
        with client.oauth2.with_streaming_response.get_token(
            grant_type="refresh_token",
            refresh_token="refresh_token",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            oauth2 = response.parse()
            assert_matches_type(Oauth2GetTokenResponse, oauth2, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncOauth2:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_get_token_overload_1(self, async_client: AsyncSlashSDK) -> None:
        oauth2 = await async_client.oauth2.get_token(
            code="code",
            grant_type="authorization_code",
            redirect_uri="redirect_uri",
        )
        assert_matches_type(Oauth2GetTokenResponse, oauth2, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_get_token_with_all_params_overload_1(self, async_client: AsyncSlashSDK) -> None:
        oauth2 = await async_client.oauth2.get_token(
            code="code",
            grant_type="authorization_code",
            redirect_uri="redirect_uri",
            code_verifier="code_verifier",
            prompt="prompt",
            scope="scope",
        )
        assert_matches_type(Oauth2GetTokenResponse, oauth2, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_get_token_overload_1(self, async_client: AsyncSlashSDK) -> None:
        response = await async_client.oauth2.with_raw_response.get_token(
            code="code",
            grant_type="authorization_code",
            redirect_uri="redirect_uri",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        oauth2 = await response.parse()
        assert_matches_type(Oauth2GetTokenResponse, oauth2, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_get_token_overload_1(self, async_client: AsyncSlashSDK) -> None:
        async with async_client.oauth2.with_streaming_response.get_token(
            code="code",
            grant_type="authorization_code",
            redirect_uri="redirect_uri",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            oauth2 = await response.parse()
            assert_matches_type(Oauth2GetTokenResponse, oauth2, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_get_token_overload_2(self, async_client: AsyncSlashSDK) -> None:
        oauth2 = await async_client.oauth2.get_token(
            grant_type="refresh_token",
            refresh_token="refresh_token",
        )
        assert_matches_type(Oauth2GetTokenResponse, oauth2, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_get_token_with_all_params_overload_2(self, async_client: AsyncSlashSDK) -> None:
        oauth2 = await async_client.oauth2.get_token(
            grant_type="refresh_token",
            refresh_token="refresh_token",
            code="code",
            code_verifier="code_verifier",
            prompt="prompt",
            redirect_uri="redirect_uri",
            scope="scope",
        )
        assert_matches_type(Oauth2GetTokenResponse, oauth2, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_get_token_overload_2(self, async_client: AsyncSlashSDK) -> None:
        response = await async_client.oauth2.with_raw_response.get_token(
            grant_type="refresh_token",
            refresh_token="refresh_token",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        oauth2 = await response.parse()
        assert_matches_type(Oauth2GetTokenResponse, oauth2, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_get_token_overload_2(self, async_client: AsyncSlashSDK) -> None:
        async with async_client.oauth2.with_streaming_response.get_token(
            grant_type="refresh_token",
            refresh_token="refresh_token",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            oauth2 = await response.parse()
            assert_matches_type(Oauth2GetTokenResponse, oauth2, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["CustomerRetrieveCurrentResponse"]


class CustomerRetrieveCurrentResponse(BaseModel):
    customer_id: str = FieldInfo(alias="customerId")



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .loan_account import LoanAccount as LoanAccount
from .deposit_account import DepositAccount as DepositAccount
from .investment_account import InvestmentAccount as InvestmentAccount
from .account_list_params import AccountListParams as AccountListParams
from .account_list_response import AccountListResponse as AccountListResponse
from .line_of_credit_account import LineOfCreditAccount as LineOfCreditAccount
from .account_retrieve_response import AccountRetrieveResponse as AccountRetrieveResponse
from .account_list_transactions_params import AccountListTransactionsParams as AccountListTransactionsParams
from .account_retrieve_contact_response import AccountRetrieveContactResponse as AccountRetrieveContactResponse
from .account_list_transactions_response import AccountListTransactionsResponse as AccountListTransactionsResponse
from .customer_retrieve_current_response import CustomerRetrieveCurrentResponse as CustomerRetrieveCurrentResponse
from .account_retrieve_payment_networks_params import (
    AccountRetrievePaymentNetworksParams as AccountRetrievePaymentNetworksParams,
)
from .account_retrieve_payment_networks_response import (
    AccountRetrievePaymentNetworksResponse as AccountRetrievePaymentNetworksResponse,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    BinaryAPIResponse,
    AsyncBinaryAPIResponse,
    StreamedBinaryAPIResponse,
    AsyncStreamedBinaryAPIResponse,
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    to_custom_raw_response_wrapper,
    async_to_streamed_response_wrapper,
    to_custom_streamed_response_wrapper,
    async_to_custom_raw_response_wrapper,
    async_to_custom_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.fdx.accounts import statement_list_params
from ....types.fdx.accounts.statement_list_response import StatementListResponse

__all__ = ["StatementsResource", "AsyncStatementsResource"]


class StatementsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> StatementsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#accessing-raw-response-data-eg-headers
        """
        return StatementsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> StatementsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#with_streaming_response
        """
        return StatementsResourceWithStreamingResponse(self)

    def list(
        self,
        account_id: str,
        *,
        end_time: str | Omit = omit,
        limit: str | Omit = omit,
        offset: str | Omit = omit,
        start_time: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> StatementListResponse:
        """
        Search for statements

        Args:
          end_time: End time for use in retrieval of statements (ISO 8601).

          limit: The number of items to return (default 100, max 1000)

          offset: The ID of the last item in the previous page of results

          start_time: Start time for use in retrieval of statements (ISO 8601).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            f"/fdx/accounts/{account_id}/statements",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "end_time": end_time,
                        "limit": limit,
                        "offset": offset,
                        "start_time": start_time,
                    },
                    statement_list_params.StatementListParams,
                ),
            ),
            cast_to=StatementListResponse,
        )

    def retrieve_pdf(
        self,
        statement_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BinaryAPIResponse:
        """
        Get account statement PDF.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not statement_id:
            raise ValueError(f"Expected a non-empty value for `statement_id` but received {statement_id!r}")
        extra_headers = {"Accept": "application/pdf", **(extra_headers or {})}
        return self._get(
            f"/fdx/accounts/{account_id}/statements/{statement_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BinaryAPIResponse,
        )


class AsyncStatementsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncStatementsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncStatementsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncStatementsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#with_streaming_response
        """
        return AsyncStatementsResourceWithStreamingResponse(self)

    async def list(
        self,
        account_id: str,
        *,
        end_time: str | Omit = omit,
        limit: str | Omit = omit,
        offset: str | Omit = omit,
        start_time: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> StatementListResponse:
        """
        Search for statements

        Args:
          end_time: End time for use in retrieval of statements (ISO 8601).

          limit: The number of items to return (default 100, max 1000)

          offset: The ID of the last item in the previous page of results

          start_time: Start time for use in retrieval of statements (ISO 8601).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            f"/fdx/accounts/{account_id}/statements",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "end_time": end_time,
                        "limit": limit,
                        "offset": offset,
                        "start_time": start_time,
                    },
                    statement_list_params.StatementListParams,
                ),
            ),
            cast_to=StatementListResponse,
        )

    async def retrieve_pdf(
        self,
        statement_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncBinaryAPIResponse:
        """
        Get account statement PDF.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not statement_id:
            raise ValueError(f"Expected a non-empty value for `statement_id` but received {statement_id!r}")
        extra_headers = {"Accept": "application/pdf", **(extra_headers or {})}
        return await self._get(
            f"/fdx/accounts/{account_id}/statements/{statement_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AsyncBinaryAPIResponse,
        )


class StatementsResourceWithRawResponse:
    def __init__(self, statements: StatementsResource) -> None:
        self._statements = statements

        self.list = to_raw_response_wrapper(
            statements.list,
        )
        self.retrieve_pdf = to_custom_raw_response_wrapper(
            statements.retrieve_pdf,
            BinaryAPIResponse,
        )


class AsyncStatementsResourceWithRawResponse:
    def __init__(self, statements: AsyncStatementsResource) -> None:
        self._statements = statements

        self.list = async_to_raw_response_wrapper(
            statements.list,
        )
        self.retrieve_pdf = async_to_custom_raw_response_wrapper(
            statements.retrieve_pdf,
            AsyncBinaryAPIResponse,
        )


class StatementsResourceWithStreamingResponse:
    def __init__(self, statements: StatementsResource) -> None:
        self._statements = statements

        self.list = to_streamed_response_wrapper(
            statements.list,
        )
        self.retrieve_pdf = to_custom_streamed_response_wrapper(
            statements.retrieve_pdf,
            StreamedBinaryAPIResponse,
        )


class AsyncStatementsResourceWithStreamingResponse:
    def __init__(self, statements: AsyncStatementsResource) -> None:
        self._statements = statements

        self.list = async_to_streamed_response_wrapper(
            statements.list,
        )
        self.retrieve_pdf = async_to_custom_streamed_response_wrapper(
            statements.retrieve_pdf,
            AsyncStreamedBinaryAPIResponse,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..types import crypto_create_offramp_params
from .._types import Body, Query, Headers, NotGiven, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.crypto_create_offramp_response import CryptoCreateOfframpResponse

__all__ = ["CryptoResource", "AsyncCryptoResource"]


class CryptoResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> CryptoResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#accessing-raw-response-data-eg-headers
        """
        return CryptoResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CryptoResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#with_streaming_response
        """
        return CryptoResourceWithStreamingResponse(self)

    def create_offramp(
        self,
        *,
        currency: Literal["usdt", "usdc"],
        payment_rail: Literal["ach", "wire"],
        virtual_account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CryptoCreateOfframpResponse:
        """
        Retrieve a list of crypto offramp options

        Args:
          virtual_account_id: The id of the virtual account to offramp funds to.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/crypto/offramp",
            body=maybe_transform(
                {
                    "currency": currency,
                    "payment_rail": payment_rail,
                    "virtual_account_id": virtual_account_id,
                },
                crypto_create_offramp_params.CryptoCreateOfframpParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CryptoCreateOfframpResponse,
        )


class AsyncCryptoResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncCryptoResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCryptoResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCryptoResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#with_streaming_response
        """
        return AsyncCryptoResourceWithStreamingResponse(self)

    async def create_offramp(
        self,
        *,
        currency: Literal["usdt", "usdc"],
        payment_rail: Literal["ach", "wire"],
        virtual_account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CryptoCreateOfframpResponse:
        """
        Retrieve a list of crypto offramp options

        Args:
          virtual_account_id: The id of the virtual account to offramp funds to.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/crypto/offramp",
            body=await async_maybe_transform(
                {
                    "currency": currency,
                    "payment_rail": payment_rail,
                    "virtual_account_id": virtual_account_id,
                },
                crypto_create_offramp_params.CryptoCreateOfframpParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CryptoCreateOfframpResponse,
        )


class CryptoResourceWithRawResponse:
    def __init__(self, crypto: CryptoResource) -> None:
        self._crypto = crypto

        self.create_offramp = to_raw_response_wrapper(
            crypto.create_offramp,
        )


class AsyncCryptoResourceWithRawResponse:
    def __init__(self, crypto: AsyncCryptoResource) -> None:
        self._crypto = crypto

        self.create_offramp = async_to_raw_response_wrapper(
            crypto.create_offramp,
        )


class CryptoResourceWithStreamingResponse:
    def __init__(self, crypto: CryptoResource) -> None:
        self._crypto = crypto

        self.create_offramp = to_streamed_response_wrapper(
            crypto.create_offramp,
        )


class AsyncCryptoResourceWithStreamingResponse:
    def __init__(self, crypto: AsyncCryptoResource) -> None:
        self._crypto = crypto

        self.create_offramp = async_to_streamed_response_wrapper(
            crypto.create_offramp,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo
from .commission_details_param import CommissionDetailsParam

__all__ = ["VirtualAccountCreateParams"]


class VirtualAccountCreateParams(TypedDict, total=False):
    account_id: Required[Annotated[str, PropertyInfo(alias="accountId")]]
    """The account ID the virtual account should be created under."""

    name: Required[str]
    """The name of the virtual account"""

    commission_details: Annotated[CommissionDetailsParam, PropertyInfo(alias="commissionDetails")]
    """
    Specifies how much of the virtual account's incoming funds should be diverted to
    the primary account
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Optional
from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo
from .card_status import CardStatus
from .card.spending_constraint_param import SpendingConstraintParam

__all__ = ["CardUpdateParams"]


class CardUpdateParams(TypedDict, total=False):
    card_group_id: Annotated[Optional[str], PropertyInfo(alias="cardGroupId")]
    """Explicitly set this value to null to remove the card from a group.

    Omitting this field entirely will not affect the group the card belongs to.
    """

    name: str

    spending_constraint: Annotated[Optional[SpendingConstraintParam], PropertyInfo(alias="spendingConstraint")]
    """
    Explicitly set this value to null to remove all card level spending constraints.
    """

    status: CardStatus

    user_data: Annotated[Optional[Dict[str, object]], PropertyInfo(alias="userData")]
    """Arbitrary information that can be attached to the card.

    This should be a JSON object and cannot exceed 4kb.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union, Optional
from typing_extensions import Literal, TypeAlias

from .._models import BaseModel

__all__ = ["PaySendResponse", "UnionMember0", "UnionMember1"]


class UnionMember0(BaseModel):
    success: Literal[True]

    redirect: Optional[str] = None


class UnionMember1(BaseModel):
    error: str

    success: Literal[False]


PaySendResponse: TypeAlias = Union[UnionMember0, UnionMember1]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Any, cast

import httpx

from ..types import pay_send_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.slash_handle import SlashHandle
from ..types.pay_send_response import PaySendResponse

__all__ = ["PayResource", "AsyncPayResource"]


class PayResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> PayResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#accessing-raw-response-data-eg-headers
        """
        return PayResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PayResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#with_streaming_response
        """
        return PayResourceWithStreamingResponse(self)

    def retrieve(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SlashHandle:
        """Retrieve your pay by slash information."""
        return self._get(
            "/pay",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SlashHandle,
        )

    def send(
        self,
        *,
        amount_cents: float,
        slash_handle: str,
        legal_entity_id: str | Omit = omit,
        source_slash_handle_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PaySendResponse:
        """
        Send money to a slash handle

        Args:
          amount_cents: The amount of money to send in cents.

          slash_handle: The username of the SlashHandle to send money to. You can get this by asking
              your recipient for their SlashHandle.

          legal_entity_id: The ID of the LegalEntity to send money from. You can get this by calling
              `GET /legal-entity`. This field or `slashHandleId` is required unless you are
              authenticating via API key.

          source_slash_handle_id: The ID of the SlashHandle to send money from. You can get this by calling
              `GET /slash-handle`. This field or `legalEntityId` is required unless you are
              authenticating via API key.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return cast(
            PaySendResponse,
            self._post(
                "/pay",
                body=maybe_transform(
                    {
                        "amount_cents": amount_cents,
                        "slash_handle": slash_handle,
                        "legal_entity_id": legal_entity_id,
                        "source_slash_handle_id": source_slash_handle_id,
                    },
                    pay_send_params.PaySendParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(Any, PaySendResponse),  # Union types cannot be passed in as arguments in the type system
            ),
        )


class AsyncPayResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncPayResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPayResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPayResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#with_streaming_response
        """
        return AsyncPayResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SlashHandle:
        """Retrieve your pay by slash information."""
        return await self._get(
            "/pay",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SlashHandle,
        )

    async def send(
        self,
        *,
        amount_cents: float,
        slash_handle: str,
        legal_entity_id: str | Omit = omit,
        source_slash_handle_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PaySendResponse:
        """
        Send money to a slash handle

        Args:
          amount_cents: The amount of money to send in cents.

          slash_handle: The username of the SlashHandle to send money to. You can get this by asking
              your recipient for their SlashHandle.

          legal_entity_id: The ID of the LegalEntity to send money from. You can get this by calling
              `GET /legal-entity`. This field or `slashHandleId` is required unless you are
              authenticating via API key.

          source_slash_handle_id: The ID of the SlashHandle to send money from. You can get this by calling
              `GET /slash-handle`. This field or `legalEntityId` is required unless you are
              authenticating via API key.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return cast(
            PaySendResponse,
            await self._post(
                "/pay",
                body=await async_maybe_transform(
                    {
                        "amount_cents": amount_cents,
                        "slash_handle": slash_handle,
                        "legal_entity_id": legal_entity_id,
                        "source_slash_handle_id": source_slash_handle_id,
                    },
                    pay_send_params.PaySendParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(Any, PaySendResponse),  # Union types cannot be passed in as arguments in the type system
            ),
        )


class PayResourceWithRawResponse:
    def __init__(self, pay: PayResource) -> None:
        self._pay = pay

        self.retrieve = to_raw_response_wrapper(
            pay.retrieve,
        )
        self.send = to_raw_response_wrapper(
            pay.send,
        )


class AsyncPayResourceWithRawResponse:
    def __init__(self, pay: AsyncPayResource) -> None:
        self._pay = pay

        self.retrieve = async_to_raw_response_wrapper(
            pay.retrieve,
        )
        self.send = async_to_raw_response_wrapper(
            pay.send,
        )


class PayResourceWithStreamingResponse:
    def __init__(self, pay: PayResource) -> None:
        self._pay = pay

        self.retrieve = to_streamed_response_wrapper(
            pay.retrieve,
        )
        self.send = to_streamed_response_wrapper(
            pay.send,
        )


class AsyncPayResourceWithStreamingResponse:
    def __init__(self, pay: AsyncPayResource) -> None:
        self._pay = pay

        self.retrieve = async_to_streamed_response_wrapper(
            pay.retrieve,
        )
        self.send = async_to_streamed_response_wrapper(
            pay.send,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["UserInfo"]


class UserInfo(BaseModel):
    email: str

    name: str
    """The user's full name"""

    sub: str
    """The user's ID"""

    phone_number: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel
from .pagination_response import PaginationResponse

__all__ = ["MerchantCategoryListResponse", "Item"]


class Item(BaseModel):
    id: str

    name: str


class MerchantCategoryListResponse(BaseModel):
    items: List[Item]

    metadata: Optional[PaginationResponse] = None
    """Response sent when requesting a list of data"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
import base64
from typing import TYPE_CHECKING, Any, Mapping
from typing_extensions import Self, override

import httpx

from . import _exceptions
from ._qs import Querystring
from ._types import (
    Omit,
    Headers,
    Timeout,
    NotGiven,
    Transport,
    ProxiesTypes,
    RequestOptions,
    not_given,
)
from ._utils import is_given, get_async_library
from ._compat import cached_property
from ._version import __version__
from ._streaming import Stream as Stream, AsyncStream as AsyncStream
from ._exceptions import APIStatusError
from ._base_client import (
    DEFAULT_MAX_RETRIES,
    SyncAPIClient,
    AsyncAPIClient,
)

if TYPE_CHECKING:
    from .resources import (
        fdx,
        pay,
        card,
        crypto,
        oauth2,
        account,
        webhook,
        merchant,
        transfer,
        card_group,
        well_known,
        transaction,
        card_product,
        legal_entity,
        slash_handle,
        virtual_account,
        developer_account,
        merchant_category,
        developer_application,
    )
    from .resources.pay import PayResource, AsyncPayResource
    from .resources.crypto import CryptoResource, AsyncCryptoResource
    from .resources.account import AccountResource, AsyncAccountResource
    from .resources.fdx.fdx import FdxResource, AsyncFdxResource
    from .resources.webhook import WebhookResource, AsyncWebhookResource
    from .resources.merchant import MerchantResource, AsyncMerchantResource
    from .resources.transfer import TransferResource, AsyncTransferResource
    from .resources.card.card import CardResource, AsyncCardResource
    from .resources.well_known import WellKnownResource, AsyncWellKnownResource
    from .resources.transaction import TransactionResource, AsyncTransactionResource
    from .resources.card_product import CardProductResource, AsyncCardProductResource
    from .resources.legal_entity import LegalEntityResource, AsyncLegalEntityResource
    from .resources.slash_handle import SlashHandleResource, AsyncSlashHandleResource
    from .resources.oauth2.oauth2 import Oauth2Resource, AsyncOauth2Resource
    from .resources.virtual_account import VirtualAccountResource, AsyncVirtualAccountResource
    from .resources.developer_account import DeveloperAccountResource, AsyncDeveloperAccountResource
    from .resources.merchant_category import MerchantCategoryResource, AsyncMerchantCategoryResource
    from .resources.card_group.card_group import CardGroupResource, AsyncCardGroupResource
    from .resources.developer_application import DeveloperApplicationResource, AsyncDeveloperApplicationResource

__all__ = [
    "Timeout",
    "Transport",
    "ProxiesTypes",
    "RequestOptions",
    "SlashSDK",
    "AsyncSlashSDK",
    "Client",
    "AsyncClient",
]


class SlashSDK(SyncAPIClient):
    # client options
    api_key: str | None
    bearer_token: str | None
    username: str | None
    password: str | None

    def __init__(
        self,
        *,
        api_key: str | None = None,
        bearer_token: str | None = None,
        username: str | None = None,
        password: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#client) for more details.
        http_client: httpx.Client | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new synchronous SlashSDK client instance.

        This automatically infers the following arguments from their corresponding environment variables if they are not provided:
        - `api_key` from `SLASH_SDK_API_KEY`
        - `bearer_token` from `SLASH_SDK_BEARER_TOKEN`
        - `username` from `SLASH_SDK_USERNAME`
        - `password` from `SLASH_SDK_PASSWORD`
        """
        if api_key is None:
            api_key = os.environ.get("SLASH_SDK_API_KEY")
        self.api_key = api_key

        if bearer_token is None:
            bearer_token = os.environ.get("SLASH_SDK_BEARER_TOKEN")
        self.bearer_token = bearer_token

        if username is None:
            username = os.environ.get("SLASH_SDK_USERNAME")
        self.username = username

        if password is None:
            password = os.environ.get("SLASH_SDK_PASSWORD")
        self.password = password

        if base_url is None:
            base_url = os.environ.get("SLASH_SDK_BASE_URL")
        if base_url is None:
            base_url = f"https://api.joinslash.com"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

    @cached_property
    def legal_entity(self) -> LegalEntityResource:
        from .resources.legal_entity import LegalEntityResource

        return LegalEntityResource(self)

    @cached_property
    def account(self) -> AccountResource:
        from .resources.account import AccountResource

        return AccountResource(self)

    @cached_property
    def virtual_account(self) -> VirtualAccountResource:
        from .resources.virtual_account import VirtualAccountResource

        return VirtualAccountResource(self)

    @cached_property
    def transaction(self) -> TransactionResource:
        from .resources.transaction import TransactionResource

        return TransactionResource(self)

    @cached_property
    def transfer(self) -> TransferResource:
        from .resources.transfer import TransferResource

        return TransferResource(self)

    @cached_property
    def card(self) -> CardResource:
        from .resources.card import CardResource

        return CardResource(self)

    @cached_property
    def card_group(self) -> CardGroupResource:
        from .resources.card_group import CardGroupResource

        return CardGroupResource(self)

    @cached_property
    def card_product(self) -> CardProductResource:
        from .resources.card_product import CardProductResource

        return CardProductResource(self)

    @cached_property
    def slash_handle(self) -> SlashHandleResource:
        from .resources.slash_handle import SlashHandleResource

        return SlashHandleResource(self)

    @cached_property
    def pay(self) -> PayResource:
        from .resources.pay import PayResource

        return PayResource(self)

    @cached_property
    def webhook(self) -> WebhookResource:
        from .resources.webhook import WebhookResource

        return WebhookResource(self)

    @cached_property
    def merchant(self) -> MerchantResource:
        from .resources.merchant import MerchantResource

        return MerchantResource(self)

    @cached_property
    def merchant_category(self) -> MerchantCategoryResource:
        from .resources.merchant_category import MerchantCategoryResource

        return MerchantCategoryResource(self)

    @cached_property
    def developer_account(self) -> DeveloperAccountResource:
        from .resources.developer_account import DeveloperAccountResource

        return DeveloperAccountResource(self)

    @cached_property
    def developer_application(self) -> DeveloperApplicationResource:
        from .resources.developer_application import DeveloperApplicationResource

        return DeveloperApplicationResource(self)

    @cached_property
    def well_known(self) -> WellKnownResource:
        from .resources.well_known import WellKnownResource

        return WellKnownResource(self)

    @cached_property
    def oauth2(self) -> Oauth2Resource:
        from .resources.oauth2 import Oauth2Resource

        return Oauth2Resource(self)

    @cached_property
    def fdx(self) -> FdxResource:
        from .resources.fdx import FdxResource

        return FdxResource(self)

    @cached_property
    def crypto(self) -> CryptoResource:
        from .resources.crypto import CryptoResource

        return CryptoResource(self)

    @cached_property
    def with_raw_response(self) -> SlashSDKWithRawResponse:
        return SlashSDKWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SlashSDKWithStreamedResponse:
        return SlashSDKWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        return {**self._api_key, **self._bearer, **self._developer_application}

    @property
    def _api_key(self) -> dict[str, str]:
        api_key = self.api_key
        if api_key is None:
            return {}
        return {"X-API-Key": api_key}

    @property
    def _bearer(self) -> dict[str, str]:
        bearer_token = self.bearer_token
        if bearer_token is None:
            return {}
        return {"Authorization": f"Bearer {bearer_token}"}

    @property
    def _developer_application(self) -> dict[str, str]:
        if self.username is None:
            return {}
        if self.password is None:
            return {}
        credentials = f"{self.username}:{self.password}".encode("ascii")
        header = f"Basic {base64.b64encode(credentials).decode('ascii')}"
        return {"Authorization": header}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": "false",
            **self._custom_headers,
        }

    @override
    def _validate_headers(self, headers: Headers, custom_headers: Headers) -> None:
        if headers.get("X-API-Key") or isinstance(custom_headers.get("X-API-Key"), Omit):
            return

        if headers.get("Authorization") or isinstance(custom_headers.get("Authorization"), Omit):
            return

        raise TypeError(
            '"Could not resolve authentication method. Expected one of api_key, bearer_token, username or password to be set. Or for one of the `X-API-Key`, `Authorization` or `Authorization` headers to be explicitly omitted"'
        )

    def copy(
        self,
        *,
        api_key: str | None = None,
        bearer_token: str | None = None,
        username: str | None = None,
        password: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.Client | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            bearer_token=bearer_token or self.bearer_token,
            username=username or self.username,
            password=password or self.password,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class AsyncSlashSDK(AsyncAPIClient):
    # client options
    api_key: str | None
    bearer_token: str | None
    username: str | None
    password: str | None

    def __init__(
        self,
        *,
        api_key: str | None = None,
        bearer_token: str | None = None,
        username: str | None = None,
        password: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultAsyncHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#asyncclient) for more details.
        http_client: httpx.AsyncClient | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new async AsyncSlashSDK client instance.

        This automatically infers the following arguments from their corresponding environment variables if they are not provided:
        - `api_key` from `SLASH_SDK_API_KEY`
        - `bearer_token` from `SLASH_SDK_BEARER_TOKEN`
        - `username` from `SLASH_SDK_USERNAME`
        - `password` from `SLASH_SDK_PASSWORD`
        """
        if api_key is None:
            api_key = os.environ.get("SLASH_SDK_API_KEY")
        self.api_key = api_key

        if bearer_token is None:
            bearer_token = os.environ.get("SLASH_SDK_BEARER_TOKEN")
        self.bearer_token = bearer_token

        if username is None:
            username = os.environ.get("SLASH_SDK_USERNAME")
        self.username = username

        if password is None:
            password = os.environ.get("SLASH_SDK_PASSWORD")
        self.password = password

        if base_url is None:
            base_url = os.environ.get("SLASH_SDK_BASE_URL")
        if base_url is None:
            base_url = f"https://api.joinslash.com"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

    @cached_property
    def legal_entity(self) -> AsyncLegalEntityResource:
        from .resources.legal_entity import AsyncLegalEntityResource

        return AsyncLegalEntityResource(self)

    @cached_property
    def account(self) -> AsyncAccountResource:
        from .resources.account import AsyncAccountResource

        return AsyncAccountResource(self)

    @cached_property
    def virtual_account(self) -> AsyncVirtualAccountResource:
        from .resources.virtual_account import AsyncVirtualAccountResource

        return AsyncVirtualAccountResource(self)

    @cached_property
    def transaction(self) -> AsyncTransactionResource:
        from .resources.transaction import AsyncTransactionResource

        return AsyncTransactionResource(self)

    @cached_property
    def transfer(self) -> AsyncTransferResource:
        from .resources.transfer import AsyncTransferResource

        return AsyncTransferResource(self)

    @cached_property
    def card(self) -> AsyncCardResource:
        from .resources.card import AsyncCardResource

        return AsyncCardResource(self)

    @cached_property
    def card_group(self) -> AsyncCardGroupResource:
        from .resources.card_group import AsyncCardGroupResource

        return AsyncCardGroupResource(self)

    @cached_property
    def card_product(self) -> AsyncCardProductResource:
        from .resources.card_product import AsyncCardProductResource

        return AsyncCardProductResource(self)

    @cached_property
    def slash_handle(self) -> AsyncSlashHandleResource:
        from .resources.slash_handle import AsyncSlashHandleResource

        return AsyncSlashHandleResource(self)

    @cached_property
    def pay(self) -> AsyncPayResource:
        from .resources.pay import AsyncPayResource

        return AsyncPayResource(self)

    @cached_property
    def webhook(self) -> AsyncWebhookResource:
        from .resources.webhook import AsyncWebhookResource

        return AsyncWebhookResource(self)

    @cached_property
    def merchant(self) -> AsyncMerchantResource:
        from .resources.merchant import AsyncMerchantResource

        return AsyncMerchantResource(self)

    @cached_property
    def merchant_category(self) -> AsyncMerchantCategoryResource:
        from .resources.merchant_category import AsyncMerchantCategoryResource

        return AsyncMerchantCategoryResource(self)

    @cached_property
    def developer_account(self) -> AsyncDeveloperAccountResource:
        from .resources.developer_account import AsyncDeveloperAccountResource

        return AsyncDeveloperAccountResource(self)

    @cached_property
    def developer_application(self) -> AsyncDeveloperApplicationResource:
        from .resources.developer_application import AsyncDeveloperApplicationResource

        return AsyncDeveloperApplicationResource(self)

    @cached_property
    def well_known(self) -> AsyncWellKnownResource:
        from .resources.well_known import AsyncWellKnownResource

        return AsyncWellKnownResource(self)

    @cached_property
    def oauth2(self) -> AsyncOauth2Resource:
        from .resources.oauth2 import AsyncOauth2Resource

        return AsyncOauth2Resource(self)

    @cached_property
    def fdx(self) -> AsyncFdxResource:
        from .resources.fdx import AsyncFdxResource

        return AsyncFdxResource(self)

    @cached_property
    def crypto(self) -> AsyncCryptoResource:
        from .resources.crypto import AsyncCryptoResource

        return AsyncCryptoResource(self)

    @cached_property
    def with_raw_response(self) -> AsyncSlashSDKWithRawResponse:
        return AsyncSlashSDKWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSlashSDKWithStreamedResponse:
        return AsyncSlashSDKWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        return {**self._api_key, **self._bearer, **self._developer_application}

    @property
    def _api_key(self) -> dict[str, str]:
        api_key = self.api_key
        if api_key is None:
            return {}
        return {"X-API-Key": api_key}

    @property
    def _bearer(self) -> dict[str, str]:
        bearer_token = self.bearer_token
        if bearer_token is None:
            return {}
        return {"Authorization": f"Bearer {bearer_token}"}

    @property
    def _developer_application(self) -> dict[str, str]:
        if self.username is None:
            return {}
        if self.password is None:
            return {}
        credentials = f"{self.username}:{self.password}".encode("ascii")
        header = f"Basic {base64.b64encode(credentials).decode('ascii')}"
        return {"Authorization": header}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": f"async:{get_async_library()}",
            **self._custom_headers,
        }

    @override
    def _validate_headers(self, headers: Headers, custom_headers: Headers) -> None:
        if headers.get("X-API-Key") or isinstance(custom_headers.get("X-API-Key"), Omit):
            return

        if headers.get("Authorization") or isinstance(custom_headers.get("Authorization"), Omit):
            return

        raise TypeError(
            '"Could not resolve authentication method. Expected one of api_key, bearer_token, username or password to be set. Or for one of the `X-API-Key`, `Authorization` or `Authorization` headers to be explicitly omitted"'
        )

    def copy(
        self,
        *,
        api_key: str | None = None,
        bearer_token: str | None = None,
        username: str | None = None,
        password: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.AsyncClient | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            bearer_token=bearer_token or self.bearer_token,
            username=username or self.username,
            password=password or self.password,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class SlashSDKWithRawResponse:
    _client: SlashSDK

    def __init__(self, client: SlashSDK) -> None:
        self._client = client

    @cached_property
    def legal_entity(self) -> legal_entity.LegalEntityResourceWithRawResponse:
        from .resources.legal_entity import LegalEntityResourceWithRawResponse

        return LegalEntityResourceWithRawResponse(self._client.legal_entity)

    @cached_property
    def account(self) -> account.AccountResourceWithRawResponse:
        from .resources.account import AccountResourceWithRawResponse

        return AccountResourceWithRawResponse(self._client.account)

    @cached_property
    def virtual_account(self) -> virtual_account.VirtualAccountResourceWithRawResponse:
        from .resources.virtual_account import VirtualAccountResourceWithRawResponse

        return VirtualAccountResourceWithRawResponse(self._client.virtual_account)

    @cached_property
    def transaction(self) -> transaction.TransactionResourceWithRawResponse:
        from .resources.transaction import TransactionResourceWithRawResponse

        return TransactionResourceWithRawResponse(self._client.transaction)

    @cached_property
    def transfer(self) -> transfer.TransferResourceWithRawResponse:
        from .resources.transfer import TransferResourceWithRawResponse

        return TransferResourceWithRawResponse(self._client.transfer)

    @cached_property
    def card(self) -> card.CardResourceWithRawResponse:
        from .resources.card import CardResourceWithRawResponse

        return CardResourceWithRawResponse(self._client.card)

    @cached_property
    def card_group(self) -> card_group.CardGroupResourceWithRawResponse:
        from .resources.card_group import CardGroupResourceWithRawResponse

        return CardGroupResourceWithRawResponse(self._client.card_group)

    @cached_property
    def card_product(self) -> card_product.CardProductResourceWithRawResponse:
        from .resources.card_product import CardProductResourceWithRawResponse

        return CardProductResourceWithRawResponse(self._client.card_product)

    @cached_property
    def slash_handle(self) -> slash_handle.SlashHandleResourceWithRawResponse:
        from .resources.slash_handle import SlashHandleResourceWithRawResponse

        return SlashHandleResourceWithRawResponse(self._client.slash_handle)

    @cached_property
    def pay(self) -> pay.PayResourceWithRawResponse:
        from .resources.pay import PayResourceWithRawResponse

        return PayResourceWithRawResponse(self._client.pay)

    @cached_property
    def webhook(self) -> webhook.WebhookResourceWithRawResponse:
        from .resources.webhook import WebhookResourceWithRawResponse

        return WebhookResourceWithRawResponse(self._client.webhook)

    @cached_property
    def merchant(self) -> merchant.MerchantResourceWithRawResponse:
        from .resources.merchant import MerchantResourceWithRawResponse

        return MerchantResourceWithRawResponse(self._client.merchant)

    @cached_property
    def merchant_category(self) -> merchant_category.MerchantCategoryResourceWithRawResponse:
        from .resources.merchant_category import MerchantCategoryResourceWithRawResponse

        return MerchantCategoryResourceWithRawResponse(self._client.merchant_category)

    @cached_property
    def developer_account(self) -> developer_account.DeveloperAccountResourceWithRawResponse:
        from .resources.developer_account import DeveloperAccountResourceWithRawResponse

        return DeveloperAccountResourceWithRawResponse(self._client.developer_account)

    @cached_property
    def developer_application(self) -> developer_application.DeveloperApplicationResourceWithRawResponse:
        from .resources.developer_application import DeveloperApplicationResourceWithRawResponse

        return DeveloperApplicationResourceWithRawResponse(self._client.developer_application)

    @cached_property
    def well_known(self) -> well_known.WellKnownResourceWithRawResponse:
        from .resources.well_known import WellKnownResourceWithRawResponse

        return WellKnownResourceWithRawResponse(self._client.well_known)

    @cached_property
    def oauth2(self) -> oauth2.Oauth2ResourceWithRawResponse:
        from .resources.oauth2 import Oauth2ResourceWithRawResponse

        return Oauth2ResourceWithRawResponse(self._client.oauth2)

    @cached_property
    def fdx(self) -> fdx.FdxResourceWithRawResponse:
        from .resources.fdx import FdxResourceWithRawResponse

        return FdxResourceWithRawResponse(self._client.fdx)

    @cached_property
    def crypto(self) -> crypto.CryptoResourceWithRawResponse:
        from .resources.crypto import CryptoResourceWithRawResponse

        return CryptoResourceWithRawResponse(self._client.crypto)


class AsyncSlashSDKWithRawResponse:
    _client: AsyncSlashSDK

    def __init__(self, client: AsyncSlashSDK) -> None:
        self._client = client

    @cached_property
    def legal_entity(self) -> legal_entity.AsyncLegalEntityResourceWithRawResponse:
        from .resources.legal_entity import AsyncLegalEntityResourceWithRawResponse

        return AsyncLegalEntityResourceWithRawResponse(self._client.legal_entity)

    @cached_property
    def account(self) -> account.AsyncAccountResourceWithRawResponse:
        from .resources.account import AsyncAccountResourceWithRawResponse

        return AsyncAccountResourceWithRawResponse(self._client.account)

    @cached_property
    def virtual_account(self) -> virtual_account.AsyncVirtualAccountResourceWithRawResponse:
        from .resources.virtual_account import AsyncVirtualAccountResourceWithRawResponse

        return AsyncVirtualAccountResourceWithRawResponse(self._client.virtual_account)

    @cached_property
    def transaction(self) -> transaction.AsyncTransactionResourceWithRawResponse:
        from .resources.transaction import AsyncTransactionResourceWithRawResponse

        return AsyncTransactionResourceWithRawResponse(self._client.transaction)

    @cached_property
    def transfer(self) -> transfer.AsyncTransferResourceWithRawResponse:
        from .resources.transfer import AsyncTransferResourceWithRawResponse

        return AsyncTransferResourceWithRawResponse(self._client.transfer)

    @cached_property
    def card(self) -> card.AsyncCardResourceWithRawResponse:
        from .resources.card import AsyncCardResourceWithRawResponse

        return AsyncCardResourceWithRawResponse(self._client.card)

    @cached_property
    def card_group(self) -> card_group.AsyncCardGroupResourceWithRawResponse:
        from .resources.card_group import AsyncCardGroupResourceWithRawResponse

        return AsyncCardGroupResourceWithRawResponse(self._client.card_group)

    @cached_property
    def card_product(self) -> card_product.AsyncCardProductResourceWithRawResponse:
        from .resources.card_product import AsyncCardProductResourceWithRawResponse

        return AsyncCardProductResourceWithRawResponse(self._client.card_product)

    @cached_property
    def slash_handle(self) -> slash_handle.AsyncSlashHandleResourceWithRawResponse:
        from .resources.slash_handle import AsyncSlashHandleResourceWithRawResponse

        return AsyncSlashHandleResourceWithRawResponse(self._client.slash_handle)

    @cached_property
    def pay(self) -> pay.AsyncPayResourceWithRawResponse:
        from .resources.pay import AsyncPayResourceWithRawResponse

        return AsyncPayResourceWithRawResponse(self._client.pay)

    @cached_property
    def webhook(self) -> webhook.AsyncWebhookResourceWithRawResponse:
        from .resources.webhook import AsyncWebhookResourceWithRawResponse

        return AsyncWebhookResourceWithRawResponse(self._client.webhook)

    @cached_property
    def merchant(self) -> merchant.AsyncMerchantResourceWithRawResponse:
        from .resources.merchant import AsyncMerchantResourceWithRawResponse

        return AsyncMerchantResourceWithRawResponse(self._client.merchant)

    @cached_property
    def merchant_category(self) -> merchant_category.AsyncMerchantCategoryResourceWithRawResponse:
        from .resources.merchant_category import AsyncMerchantCategoryResourceWithRawResponse

        return AsyncMerchantCategoryResourceWithRawResponse(self._client.merchant_category)

    @cached_property
    def developer_account(self) -> developer_account.AsyncDeveloperAccountResourceWithRawResponse:
        from .resources.developer_account import AsyncDeveloperAccountResourceWithRawResponse

        return AsyncDeveloperAccountResourceWithRawResponse(self._client.developer_account)

    @cached_property
    def developer_application(self) -> developer_application.AsyncDeveloperApplicationResourceWithRawResponse:
        from .resources.developer_application import AsyncDeveloperApplicationResourceWithRawResponse

        return AsyncDeveloperApplicationResourceWithRawResponse(self._client.developer_application)

    @cached_property
    def well_known(self) -> well_known.AsyncWellKnownResourceWithRawResponse:
        from .resources.well_known import AsyncWellKnownResourceWithRawResponse

        return AsyncWellKnownResourceWithRawResponse(self._client.well_known)

    @cached_property
    def oauth2(self) -> oauth2.AsyncOauth2ResourceWithRawResponse:
        from .resources.oauth2 import AsyncOauth2ResourceWithRawResponse

        return AsyncOauth2ResourceWithRawResponse(self._client.oauth2)

    @cached_property
    def fdx(self) -> fdx.AsyncFdxResourceWithRawResponse:
        from .resources.fdx import AsyncFdxResourceWithRawResponse

        return AsyncFdxResourceWithRawResponse(self._client.fdx)

    @cached_property
    def crypto(self) -> crypto.AsyncCryptoResourceWithRawResponse:
        from .resources.crypto import AsyncCryptoResourceWithRawResponse

        return AsyncCryptoResourceWithRawResponse(self._client.crypto)


class SlashSDKWithStreamedResponse:
    _client: SlashSDK

    def __init__(self, client: SlashSDK) -> None:
        self._client = client

    @cached_property
    def legal_entity(self) -> legal_entity.LegalEntityResourceWithStreamingResponse:
        from .resources.legal_entity import LegalEntityResourceWithStreamingResponse

        return LegalEntityResourceWithStreamingResponse(self._client.legal_entity)

    @cached_property
    def account(self) -> account.AccountResourceWithStreamingResponse:
        from .resources.account import AccountResourceWithStreamingResponse

        return AccountResourceWithStreamingResponse(self._client.account)

    @cached_property
    def virtual_account(self) -> virtual_account.VirtualAccountResourceWithStreamingResponse:
        from .resources.virtual_account import VirtualAccountResourceWithStreamingResponse

        return VirtualAccountResourceWithStreamingResponse(self._client.virtual_account)

    @cached_property
    def transaction(self) -> transaction.TransactionResourceWithStreamingResponse:
        from .resources.transaction import TransactionResourceWithStreamingResponse

        return TransactionResourceWithStreamingResponse(self._client.transaction)

    @cached_property
    def transfer(self) -> transfer.TransferResourceWithStreamingResponse:
        from .resources.transfer import TransferResourceWithStreamingResponse

        return TransferResourceWithStreamingResponse(self._client.transfer)

    @cached_property
    def card(self) -> card.CardResourceWithStreamingResponse:
        from .resources.card import CardResourceWithStreamingResponse

        return CardResourceWithStreamingResponse(self._client.card)

    @cached_property
    def card_group(self) -> card_group.CardGroupResourceWithStreamingResponse:
        from .resources.card_group import CardGroupResourceWithStreamingResponse

        return CardGroupResourceWithStreamingResponse(self._client.card_group)

    @cached_property
    def card_product(self) -> card_product.CardProductResourceWithStreamingResponse:
        from .resources.card_product import CardProductResourceWithStreamingResponse

        return CardProductResourceWithStreamingResponse(self._client.card_product)

    @cached_property
    def slash_handle(self) -> slash_handle.SlashHandleResourceWithStreamingResponse:
        from .resources.slash_handle import SlashHandleResourceWithStreamingResponse

        return SlashHandleResourceWithStreamingResponse(self._client.slash_handle)

    @cached_property
    def pay(self) -> pay.PayResourceWithStreamingResponse:
        from .resources.pay import PayResourceWithStreamingResponse

        return PayResourceWithStreamingResponse(self._client.pay)

    @cached_property
    def webhook(self) -> webhook.WebhookResourceWithStreamingResponse:
        from .resources.webhook import WebhookResourceWithStreamingResponse

        return WebhookResourceWithStreamingResponse(self._client.webhook)

    @cached_property
    def merchant(self) -> merchant.MerchantResourceWithStreamingResponse:
        from .resources.merchant import MerchantResourceWithStreamingResponse

        return MerchantResourceWithStreamingResponse(self._client.merchant)

    @cached_property
    def merchant_category(self) -> merchant_category.MerchantCategoryResourceWithStreamingResponse:
        from .resources.merchant_category import MerchantCategoryResourceWithStreamingResponse

        return MerchantCategoryResourceWithStreamingResponse(self._client.merchant_category)

    @cached_property
    def developer_account(self) -> developer_account.DeveloperAccountResourceWithStreamingResponse:
        from .resources.developer_account import DeveloperAccountResourceWithStreamingResponse

        return DeveloperAccountResourceWithStreamingResponse(self._client.developer_account)

    @cached_property
    def developer_application(self) -> developer_application.DeveloperApplicationResourceWithStreamingResponse:
        from .resources.developer_application import DeveloperApplicationResourceWithStreamingResponse

        return DeveloperApplicationResourceWithStreamingResponse(self._client.developer_application)

    @cached_property
    def well_known(self) -> well_known.WellKnownResourceWithStreamingResponse:
        from .resources.well_known import WellKnownResourceWithStreamingResponse

        return WellKnownResourceWithStreamingResponse(self._client.well_known)

    @cached_property
    def oauth2(self) -> oauth2.Oauth2ResourceWithStreamingResponse:
        from .resources.oauth2 import Oauth2ResourceWithStreamingResponse

        return Oauth2ResourceWithStreamingResponse(self._client.oauth2)

    @cached_property
    def fdx(self) -> fdx.FdxResourceWithStreamingResponse:
        from .resources.fdx import FdxResourceWithStreamingResponse

        return FdxResourceWithStreamingResponse(self._client.fdx)

    @cached_property
    def crypto(self) -> crypto.CryptoResourceWithStreamingResponse:
        from .resources.crypto import CryptoResourceWithStreamingResponse

        return CryptoResourceWithStreamingResponse(self._client.crypto)


class AsyncSlashSDKWithStreamedResponse:
    _client: AsyncSlashSDK

    def __init__(self, client: AsyncSlashSDK) -> None:
        self._client = client

    @cached_property
    def legal_entity(self) -> legal_entity.AsyncLegalEntityResourceWithStreamingResponse:
        from .resources.legal_entity import AsyncLegalEntityResourceWithStreamingResponse

        return AsyncLegalEntityResourceWithStreamingResponse(self._client.legal_entity)

    @cached_property
    def account(self) -> account.AsyncAccountResourceWithStreamingResponse:
        from .resources.account import AsyncAccountResourceWithStreamingResponse

        return AsyncAccountResourceWithStreamingResponse(self._client.account)

    @cached_property
    def virtual_account(self) -> virtual_account.AsyncVirtualAccountResourceWithStreamingResponse:
        from .resources.virtual_account import AsyncVirtualAccountResourceWithStreamingResponse

        return AsyncVirtualAccountResourceWithStreamingResponse(self._client.virtual_account)

    @cached_property
    def transaction(self) -> transaction.AsyncTransactionResourceWithStreamingResponse:
        from .resources.transaction import AsyncTransactionResourceWithStreamingResponse

        return AsyncTransactionResourceWithStreamingResponse(self._client.transaction)

    @cached_property
    def transfer(self) -> transfer.AsyncTransferResourceWithStreamingResponse:
        from .resources.transfer import AsyncTransferResourceWithStreamingResponse

        return AsyncTransferResourceWithStreamingResponse(self._client.transfer)

    @cached_property
    def card(self) -> card.AsyncCardResourceWithStreamingResponse:
        from .resources.card import AsyncCardResourceWithStreamingResponse

        return AsyncCardResourceWithStreamingResponse(self._client.card)

    @cached_property
    def card_group(self) -> card_group.AsyncCardGroupResourceWithStreamingResponse:
        from .resources.card_group import AsyncCardGroupResourceWithStreamingResponse

        return AsyncCardGroupResourceWithStreamingResponse(self._client.card_group)

    @cached_property
    def card_product(self) -> card_product.AsyncCardProductResourceWithStreamingResponse:
        from .resources.card_product import AsyncCardProductResourceWithStreamingResponse

        return AsyncCardProductResourceWithStreamingResponse(self._client.card_product)

    @cached_property
    def slash_handle(self) -> slash_handle.AsyncSlashHandleResourceWithStreamingResponse:
        from .resources.slash_handle import AsyncSlashHandleResourceWithStreamingResponse

        return AsyncSlashHandleResourceWithStreamingResponse(self._client.slash_handle)

    @cached_property
    def pay(self) -> pay.AsyncPayResourceWithStreamingResponse:
        from .resources.pay import AsyncPayResourceWithStreamingResponse

        return AsyncPayResourceWithStreamingResponse(self._client.pay)

    @cached_property
    def webhook(self) -> webhook.AsyncWebhookResourceWithStreamingResponse:
        from .resources.webhook import AsyncWebhookResourceWithStreamingResponse

        return AsyncWebhookResourceWithStreamingResponse(self._client.webhook)

    @cached_property
    def merchant(self) -> merchant.AsyncMerchantResourceWithStreamingResponse:
        from .resources.merchant import AsyncMerchantResourceWithStreamingResponse

        return AsyncMerchantResourceWithStreamingResponse(self._client.merchant)

    @cached_property
    def merchant_category(self) -> merchant_category.AsyncMerchantCategoryResourceWithStreamingResponse:
        from .resources.merchant_category import AsyncMerchantCategoryResourceWithStreamingResponse

        return AsyncMerchantCategoryResourceWithStreamingResponse(self._client.merchant_category)

    @cached_property
    def developer_account(self) -> developer_account.AsyncDeveloperAccountResourceWithStreamingResponse:
        from .resources.developer_account import AsyncDeveloperAccountResourceWithStreamingResponse

        return AsyncDeveloperAccountResourceWithStreamingResponse(self._client.developer_account)

    @cached_property
    def developer_application(self) -> developer_application.AsyncDeveloperApplicationResourceWithStreamingResponse:
        from .resources.developer_application import AsyncDeveloperApplicationResourceWithStreamingResponse

        return AsyncDeveloperApplicationResourceWithStreamingResponse(self._client.developer_application)

    @cached_property
    def well_known(self) -> well_known.AsyncWellKnownResourceWithStreamingResponse:
        from .resources.well_known import AsyncWellKnownResourceWithStreamingResponse

        return AsyncWellKnownResourceWithStreamingResponse(self._client.well_known)

    @cached_property
    def oauth2(self) -> oauth2.AsyncOauth2ResourceWithStreamingResponse:
        from .resources.oauth2 import AsyncOauth2ResourceWithStreamingResponse

        return AsyncOauth2ResourceWithStreamingResponse(self._client.oauth2)

    @cached_property
    def fdx(self) -> fdx.AsyncFdxResourceWithStreamingResponse:
        from .resources.fdx import AsyncFdxResourceWithStreamingResponse

        return AsyncFdxResourceWithStreamingResponse(self._client.fdx)

    @cached_property
    def crypto(self) -> crypto.AsyncCryptoResourceWithStreamingResponse:
        from .resources.crypto import AsyncCryptoResourceWithStreamingResponse

        return AsyncCryptoResourceWithStreamingResponse(self._client.crypto)


Client = SlashSDK

AsyncClient = AsyncSlashSDK



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel
from .pagination_response import PaginationResponse

__all__ = ["LegalEntityListResponse", "Item"]


class Item(BaseModel):
    """
    A legal entity is an entity (either a person or a business) that can own accounts.
    If you are using the API with an API key, your API key will only be associated with one legal entity, so you will not need to specify the legal entity ID in most endpoints.
    If you are using the API with an OAuth2 access token, you can filter the results of most endpoints to only show data for a specific legal entity by specifying the legal entity ID in the URL.
    """

    id: str

    name: str

    structure: Optional[
        Literal["person", "llc", "lp", "llp", "gp", "partnership", "cooperative", "ccorp", "scorp", "other"]
    ] = None
    """
    Will be empty in rare cases where the legal entity has not yet been fully
    created.
    """


class LegalEntityListResponse(BaseModel):
    items: List[Item]

    metadata: PaginationResponse
    """Response sent when requesting a list of data"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel
from .pagination_response import PaginationResponse

__all__ = ["CardProductListResponse", "Item"]


class Item(BaseModel):
    id: str

    prefix: str

    status: Literal["active", "inactive"]


class CardProductListResponse(BaseModel):
    items: List[Item]

    metadata: Optional[PaginationResponse] = None
    """Response sent when requesting a list of data"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from typing_extensions import Literal, TypeAlias

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = [
    "AccountListTransactionsResponse",
    "Transaction",
    "TransactionDepositTransaction",
    "TransactionDepositTransactionDepositTransaction",
    "TransactionLoanTransaction",
    "TransactionLoanTransactionLoanTransaction",
    "TransactionLocTransaction",
    "TransactionLocTransactionLocTransaction",
    "TransactionInvestmentTransaction",
    "TransactionInvestmentTransactionInvestmentTransaction",
    "TransactionInvestmentTransactionInvestmentTransactionFiAttribute",
    "Page",
]


class TransactionDepositTransactionDepositTransaction(BaseModel):
    amount: float
    """
    The amount of money in the account currency in that currency's standard unit
    (e.g. dollars for USD). The amount is an absolute value. The debitCreditMemo
    parameter indicates the direction of the transaction.
    """

    debit_credit_memo: Literal["DEBIT", "CREDIT", "MEMO"] = FieldInfo(alias="debitCreditMemo")
    """The posting type of a transaction.

    The transaction amount is an absolute value, and this parameter indicates the
    direction of the transaction. This will always be DEBIT or CREDIT.
    """

    description: str
    """A description of the transaction."""

    status: Literal["AUTHORIZATION", "MEMO", "PENDING", "PENDING_APPROVAL", "POSTED", "IN_REVIEW"]
    """The status of the transaction. This will always be PENDING or POSTED."""

    transaction_id: str = FieldInfo(alias="transactionId")
    """Long-term persistent identity of the transaction."""

    transaction_timestamp: str = FieldInfo(alias="transactionTimestamp")
    """The ISO 8601 date and time that the transaction was initiated."""

    category: Optional[str] = None
    """The merchant category code (MCC) of the transaction, if available."""

    check_number: Optional[str] = FieldInfo(alias="checkNumber", default=None)
    """The check number of the transaction, if available.

    Only included if this is a check transaction.
    """

    foreign_amount: Optional[float] = FieldInfo(alias="foreignAmount", default=None)
    """
    This value is only included if the transaction's currency is different than the
    account's default currency.
    """

    foreign_currency: Optional[str] = FieldInfo(alias="foreignCurrency", default=None)
    """ISO 4217 currency code.

    This value is only included if the transaction's currency is different than the
    account's default currency.
    """

    payee: Optional[str] = None

    posted_timestamp: Optional[str] = FieldInfo(alias="postedTimestamp", default=None)
    """The ISO 8601 date and time that the transaction was posted to the account.

    This will be included if status is POSTED.
    """

    reference_transaction_id: Optional[str] = FieldInfo(alias="referenceTransactionId", default=None)
    """For reverse postings, the identity of the transaction being reversed.

    For the correction transaction, the identity of the reversing post. For credit
    card posting transactions, the identity of the authorization transaction
    """

    sub_category: Optional[str] = FieldInfo(alias="subCategory", default=None)
    """Additional information about the transaction category, if available."""


class TransactionDepositTransaction(BaseModel):
    deposit_transaction: TransactionDepositTransactionDepositTransaction = FieldInfo(alias="depositTransaction")


class TransactionLoanTransactionLoanTransaction(BaseModel):
    amount: float
    """
    The amount of money in the account currency in that currency's standard unit
    (e.g. dollars for USD). The amount is an absolute value. The debitCreditMemo
    parameter indicates the direction of the transaction.
    """

    debit_credit_memo: Literal["DEBIT", "CREDIT", "MEMO"] = FieldInfo(alias="debitCreditMemo")
    """The posting type of a transaction.

    The transaction amount is an absolute value, and this parameter indicates the
    direction of the transaction. This will always be DEBIT or CREDIT.
    """

    description: str
    """A description of the transaction."""

    status: Literal["AUTHORIZATION", "MEMO", "PENDING", "IN_REVIEW", "POSTED"]
    """The status of the transaction. This will always be PENDING or POSTED."""

    transaction_id: str = FieldInfo(alias="transactionId")
    """Long-term persistent identity of the transaction."""

    transaction_timestamp: str = FieldInfo(alias="transactionTimestamp")
    """The ISO 8601 date and time that the transaction was initiated."""

    category: Optional[str] = None
    """The merchant category code (MCC) of the transaction, if available."""

    foreign_amount: Optional[float] = FieldInfo(alias="foreignAmount", default=None)
    """
    This value is only included if the transaction's currency is different than the
    account's default currency.
    """

    foreign_currency: Optional[str] = FieldInfo(alias="foreignCurrency", default=None)
    """ISO 4217 currency code.

    This value is only included if the transaction's currency is different than the
    account's default currency.
    """

    posted_timestamp: Optional[str] = FieldInfo(alias="postedTimestamp", default=None)
    """The ISO 8601 date and time that the transaction was posted to the account.

    This will be included if status is POSTED.
    """

    reference_transaction_id: Optional[str] = FieldInfo(alias="referenceTransactionId", default=None)
    """For reverse postings, the identity of the transaction being reversed.

    For the correction transaction, the identity of the reversing post. For credit
    card posting transactions, the identity of the authorization transaction
    """

    sub_category: Optional[str] = FieldInfo(alias="subCategory", default=None)
    """Additional information about the transaction category, if available."""

    transaction_type: Optional[
        Literal[
            "ADJUSTMENT",
            "FEE",
            "INTEREST",
            "PAYMENT",
            "LUMP_SUM_PAYMENT",
            "SKIP_PAYMENT",
            "DOUBLE_UP_PAYMENT",
            "PAYOFF",
        ]
    ] = FieldInfo(alias="transactionType", default=None)


class TransactionLoanTransaction(BaseModel):
    loan_transaction: TransactionLoanTransactionLoanTransaction = FieldInfo(alias="loanTransaction")


class TransactionLocTransactionLocTransaction(BaseModel):
    amount: float
    """
    The amount of money in the account currency in that currency's standard unit
    (e.g. dollars for USD). The amount is an absolute value. The debitCreditMemo
    parameter indicates the direction of the transaction.
    """

    debit_credit_memo: Literal["DEBIT", "CREDIT", "MEMO"] = FieldInfo(alias="debitCreditMemo")
    """The posting type of a transaction.

    The transaction amount is an absolute value, and this parameter indicates the
    direction of the transaction. This will always be DEBIT or CREDIT.
    """

    description: str
    """A description of the transaction."""

    status: Literal["AUTHORIZATION", "MEMO", "PENDING", "PENDING_APPROVAL", "IN_REVIEW", "POSTED"]
    """The status of the transaction. This will always be PENDING or POSTED."""

    transaction_id: str = FieldInfo(alias="transactionId")
    """Long-term persistent identity of the transaction."""

    transaction_timestamp: str = FieldInfo(alias="transactionTimestamp")
    """The ISO 8601 date and time that the transaction was initiated."""

    category: Optional[str] = None
    """The merchant category code (MCC) of the transaction, if available."""

    check_number: Optional[float] = FieldInfo(alias="checkNumber", default=None)
    """The check number of the transaction, if available.

    Only included if transactionType is CHECK.
    """

    foreign_amount: Optional[float] = FieldInfo(alias="foreignAmount", default=None)
    """
    This value is only included if the transaction's currency is different than the
    account's default currency.
    """

    foreign_currency: Optional[str] = FieldInfo(alias="foreignCurrency", default=None)
    """ISO 4217 currency code.

    This value is only included if the transaction's currency is different than the
    account's default currency.
    """

    posted_timestamp: Optional[str] = FieldInfo(alias="postedTimestamp", default=None)
    """The ISO 8601 date and time that the transaction was posted to the account.

    This will be included if status is POSTED.
    """

    reference_transaction_id: Optional[str] = FieldInfo(alias="referenceTransactionId", default=None)
    """For reverse postings, the identity of the transaction being reversed.

    For the correction transaction, the identity of the reversing post. For credit
    card posting transactions, the identity of the authorization transaction
    """

    sub_category: Optional[str] = FieldInfo(alias="subCategory", default=None)
    """Additional information about the transaction category, if available."""

    transaction_type: Optional[Literal["ADJUSTMENT", "CHECK", "FEE", "INTEREST", "PAYMENT", "WITHDRAWAL"]] = FieldInfo(
        alias="transactionType", default=None
    )


class TransactionLocTransaction(BaseModel):
    loc_transaction: TransactionLocTransactionLocTransaction = FieldInfo(alias="locTransaction")


class TransactionInvestmentTransactionInvestmentTransactionFiAttribute(BaseModel):
    name: str

    value: str


class TransactionInvestmentTransactionInvestmentTransaction(BaseModel):
    amount: float
    """
    The amount of money in the account currency in that currency's standard unit
    (e.g. dollars for USD). The amount is an absolute value. The debitCreditMemo
    parameter indicates the direction of the transaction.
    """

    debit_credit_memo: Literal["DEBIT", "CREDIT", "MEMO"] = FieldInfo(alias="debitCreditMemo")
    """The posting type of a transaction.

    The transaction amount is an absolute value, and this parameter indicates the
    direction of the transaction. This will always be DEBIT or CREDIT.
    """

    description: str
    """A description of the transaction."""

    fees: float

    status: Literal["AUTHORIZATION", "MEMO", "PENDING", "POSTED"]
    """The status of the transaction. This will always be PENDING or POSTED."""

    transaction_id: str = FieldInfo(alias="transactionId")
    """Long-term persistent identity of the transaction."""

    transaction_timestamp: str = FieldInfo(alias="transactionTimestamp")
    """The ISO 8601 date and time that the transaction was initiated."""

    category: Optional[str] = None
    """The merchant category code (MCC) of the transaction, if available."""

    commission: Optional[float] = None

    fi_attributes: Optional[List[TransactionInvestmentTransactionInvestmentTransactionFiAttribute]] = FieldInfo(
        alias="fiAttributes", default=None
    )

    foreign_amount: Optional[float] = FieldInfo(alias="foreignAmount", default=None)
    """
    This value is only included if the transaction's currency is different than the
    account's default currency.
    """

    foreign_currency: Optional[str] = FieldInfo(alias="foreignCurrency", default=None)
    """ISO 4217 currency code.

    This value is only included if the transaction's currency is different than the
    account's default currency.
    """

    posted_timestamp: Optional[str] = FieldInfo(alias="postedTimestamp", default=None)
    """The ISO 8601 date and time that the transaction was posted to the account.

    This will be included if status is POSTED.
    """

    reference_transaction_id: Optional[str] = FieldInfo(alias="referenceTransactionId", default=None)
    """For reverse postings, the identity of the transaction being reversed.

    For the correction transaction, the identity of the reversing post. For credit
    card posting transactions, the identity of the authorization transaction
    """

    security_id: Optional[str] = FieldInfo(alias="securityId", default=None)
    """The security identifier for the transaction, if applicable."""

    security_id_type: Optional[
        Literal["CINS", "CMC", "CME", "CUSIP", "ISIN", "ITSA", "NASDAQ", "SEDOL", "SICC", "VALOR", "WKN"]
    ] = FieldInfo(alias="securityIdType", default=None)

    security_type: Optional[
        Literal["BOND", "DEBT", "DIGITALASSET", "MUTUALFUND", "OPTION", "OTHER", "STOCK", "SWEEP"]
    ] = FieldInfo(alias="securityType", default=None)

    sub_category: Optional[str] = FieldInfo(alias="subCategory", default=None)
    """Additional information about the transaction category, if available."""

    symbol: Optional[str] = None
    """Ticker symbol"""

    transaction_type: Optional[
        Literal[
            "ADJUSTMENT",
            "ATM",
            "CASH",
            "CHECK",
            "CLOSURE",
            "CLOSUREOPT",
            "CONTRIBUTION",
            "DEP",
            "DEPOSIT",
            "DIRECTDEBIT",
            "DIRECTDEP",
            "DIV",
            "DIVIDEND",
            "DIVIDENDREINVEST",
            "EXPENSE",
            "FEE",
            "INCOME",
            "INTEREST",
            "INVEXPENSE",
            "JRNLFUND",
            "JRNLSEC",
            "MARGININTEREST",
            "OPTIONEXERCISE",
            "OPTIONEXPIRATION",
            "OTHER",
            "PAYMENT",
            "POS",
            "PURCHASED",
            "PURCHASEDTOCOVER",
            "PURCHASETOCLOSE",
            "PURCHASETOOPEN",
            "REINVESTOFINCOME",
            "REPEATPMT",
            "RETURNOFCAPITAL",
            "SOLD",
            "SOLDTOCLOSE",
            "SOLDTOOPEN",
            "SPLIT",
            "SRVCHG",
            "TRANSFER",
            "XFER",
        ]
    ] = FieldInfo(alias="transactionType", default=None)

    unit_price: Optional[float] = FieldInfo(alias="unitPrice", default=None)

    units: Optional[float] = None

    unit_type: Optional[Literal["CURRENCY", "SHARES"]] = FieldInfo(alias="unitType", default=None)


class TransactionInvestmentTransaction(BaseModel):
    investment_transaction: TransactionInvestmentTransactionInvestmentTransaction = FieldInfo(
        alias="investmentTransaction"
    )


Transaction: TypeAlias = Union[
    TransactionDepositTransaction,
    TransactionLoanTransaction,
    TransactionLocTransaction,
    TransactionInvestmentTransaction,
]


class Page(BaseModel):
    next_offset: Optional[str] = FieldInfo(alias="nextOffset", default=None)


class AccountListTransactionsResponse(BaseModel):
    transactions: List[Transaction]

    page: Optional[Page] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["WebhookListParams"]


class WebhookListParams(TypedDict, total=False):
    cursor: str
    """A cursor string to fetch the next page of results"""

    filter_legal_entity_id: Annotated[str, PropertyInfo(alias="filter:legalEntityId")]
    """Pass in a legal entity ID to filter for webhooks under a specific legal entity."""
